import tkinter as tk
from tkinter import simpledialog, messagebox
import schedule
import time
import threading
import json
from datetime import datetime, timedelta
import ctypes

class Task:
    def __init__(self, description, datetime_str, frequency):
        self.description = description
        self.datetime_str = datetime_str
        self.frequency = frequency

    def to_dict(self):
        return {'description': self.description, 'datetime': self.datetime_str, 'frequency': self.frequency}

    @staticmethod
    def from_dict(task_dict):
        return Task(task_dict['description'], task_dict['datetime'], task_dict['frequency'])

class TaskSchedulerApp:
    def __init__(self, root):
        self.root = root
        self.root.title("Task Scheduler")
        self.tasks = []
        self.running = True
        self.timers = []

        self.frame = tk.Frame(root)
        self.frame.pack(pady=20)

        self.task_listbox = tk.Listbox(self.frame, width=50)
        self.task_listbox.pack(side=tk.LEFT, padx=10)

        self.scrollbar = tk.Scrollbar(self.frame, orient=tk.VERTICAL, command=self.task_listbox.yview)
        self.scrollbar.pack(side=tk.RIGHT, fill=tk.Y)
        self.task_listbox.config(yscrollcommand=self.scrollbar.set)

        self.add_button = tk.Button(root, text="Add Task", command=self.add_task_dialog)
        self.add_button.pack(pady=10)
     
        self.edit_button = tk.Button(root, text="Edit Task", command=self.edit_task_dialog)
        self.edit_button.pack(pady=10)

        self.copy_button = tk.Button(root, text="Copy Task", command=self.copy_task)
        self.copy_button.pack(pady=10)


        self.delete_button = tk.Button(root, text="Delete Task", command=self.delete_task)
        self.delete_button.pack(pady=10)

        self.root = root
        self.show_messagebox_var = tk.BooleanVar(value=False)  # Checkbox state variable
                # Add the checkbox to the window
        self.checkbox = tk.Checkbutton(root, text="Show Task Reminder", variable=self.show_messagebox_var)
        self.checkbox.pack()


        self.load_tasks()
        self.run_scheduler()

        self.root.protocol("WM_DELETE_WINDOW", self.on_closing)

    def play_sound(self):
        for _ in range(3):
            ctypes.windll.kernel32.Beep(750, 300)
            time.sleep(0.3)

    def add_task_dialog(self):
        description = simpledialog.askstring("Task Description", "Enter the task description:")
        datetime_str = simpledialog.askstring("Task DateTime", "Enter the task date and time (YYYY-MM-DD HH:MM):")
        frequency = simpledialog.askstring("Task Frequency", "Enter the task frequency (e.g., '0','30 minutes', '1 hour', '1 day'):")

        if description and datetime_str and frequency:
            self.add_task(description, datetime_str, frequency)

    def add_task(self, description, datetime_str, frequency):
        task = Task(description, datetime_str, frequency)
        self.tasks.append(task)
        self.task_listbox.insert(tk.END, f"{description} at {datetime_str} every {frequency}")
        self.schedule_task(task)
        self.save_tasks()

    def delete_task(self):
        selected_task_index = self.task_listbox.curselection()
        if selected_task_index:
            self.task_listbox.delete(selected_task_index)
            self.tasks.pop(selected_task_index[0])
            self.save_tasks()
            self.reschedule_tasks()

    def edit_task_dialog(self):
        selected_task_index = self.task_listbox.curselection()
        if not selected_task_index:
            messagebox.showwarning("Edit Task", "Please select a task to edit.")
            return

        index = selected_task_index[0]
        old_task = self.tasks[index]

        new_description = simpledialog.askstring("Edit Task Description", "Enter the new task description:", initialvalue=old_task.description)
        new_datetime_str = simpledialog.askstring("Edit Task DateTime", "Enter the new task date and time (YYYY-MM-DD HH:MM):", initialvalue=old_task.datetime_str)
        new_frequency = simpledialog.askstring("Edit Task Frequency", "Enter the new task frequency (e.g., '30 minutes', '1 hour', '1 day'):", initialvalue=old_task.frequency)

        if new_description and new_datetime_str and new_frequency:
            self.tasks[index] = Task(new_description, new_datetime_str, new_frequency)
            self.task_listbox.delete(index)
            self.task_listbox.insert(index, f"{new_description} at {new_datetime_str} every {new_frequency}")
            self.reschedule_tasks()
            self.save_tasks()

    def copy_task(self):
        selected_task_index = self.task_listbox.curselection()
        if not selected_task_index:
            messagebox.showwarning("Copy Task", "Please select a task to copy.")
            return

        index = selected_task_index[0]
        old_task = self.tasks[index]

        new_description = simpledialog.askstring("Copy Task Description", "Enter the new task description:", initialvalue=old_task.description)
        new_datetime_str = simpledialog.askstring("Copy Task DateTime", "Enter the new task date and time (YYYY-MM-DD HH:MM):", initialvalue=old_task.datetime_str)
        new_frequency = simpledialog.askstring("Copy Task Frequency", "Enter the new task frequency (e.g., '30 minutes', '1 hour', '1 day'):", initialvalue=old_task.frequency)

        if new_description and new_datetime_str and new_frequency:
            self.add_task(new_description, new_datetime_str, new_frequency)


    def schedule_task(self, task):
        def job():
            for _ in range(1):
                self.play_sound()
                time.sleep(0.3)
            if self.show_messagebox_var.get():
                messagebox.showinfo("Task Reminder", f"Task: {task.description} is scheduled for now.")
            # If checkbox is unchecked, do nothing (no messagebox)

        task_time = datetime.strptime(task.datetime_str, "%Y-%m-%d %H:%M")
        now = datetime.now()

        # Обнуляем секунды и миллисекунды
        task_time = task_time.replace(second=0, microsecond=0)
        now = now.replace(second=0, microsecond=0)
        
        if now > task_time:
            task_time += timedelta(days=1)

        delay = (task_time - now).total_seconds()
        

        if task.frequency == '0':
            # Запланировать задачу на конкретное время, не повторяя
            task_time = datetime.strptime(task.datetime_str, "%Y-%m-%d %H:%M")
            now = datetime.now()
            task_time = task_time.replace(second=0, microsecond=0)
            if now > task_time:
                task_time += timedelta(days=1)

            delay = (task_time - now).total_seconds()
            timer = threading.Timer(delay, job)
            timer.start()
            self.timers.append(timer)

        else:
            if 'minute' in task.frequency:
                interval = int(task.frequency.split()[0])
                schedule.every(interval).minutes.at(":00").do(job)
            elif 'hour' in task.frequency:
                interval = int(task.frequency.split()[0])
                schedule.every(interval).hours.at(":00").do(job)
            elif 'day' in task.frequency:
                interval = int(task.frequency.split()[0])
                schedule.every(interval).days.at("00:00").do(job)

     
    def reschedule_tasks(self):
        schedule.clear()
        for timer in self.timers:
            timer.cancel()
        self.timers.clear()
        for task in self.tasks:
            self.schedule_task(task)

    def save_tasks(self):
        with open('tasks.json', 'w') as f:
            json.dump([task.to_dict() for task in self.tasks], f)

    def load_tasks(self):
        try:
            with open('tasks.json', 'r') as f:
                tasks_data = json.load(f)
                self.tasks = [Task.from_dict(task) for task in tasks_data]
                self.task_listbox.delete(0, tk.END)
                for task in self.tasks:
                    self.task_listbox.insert(tk.END, f"{task.description} at {task.datetime_str} every {task.frequency}")
                    self.schedule_task(task)
        except FileNotFoundError:
            pass

    def run_scheduler(self):
        def run():
            while self.running:
                schedule.run_pending()
                time.sleep(1)

        self.scheduler_thread = threading.Thread(target=run, daemon=True)
        self.scheduler_thread.start()

    def on_closing(self):
        self.running = False
        for timer in self.timers:
            timer.cancel()
        self.scheduler_thread.join()
        self.root.destroy()

if __name__ == "__main__":
    root = tk.Tk()
    app = TaskSchedulerApp(root)
    root.mainloop()
